@@ -52,6 +52,8 @@ except ImportError:
 
 from libqtile.log_utils import logger
 
+dbus_bus_connections = set()
+
 # Create a list to collect references to tasks so they're not garbage collected
 # before they've run
 TASKS: list[asyncio.Task[None]] = []
@@ -424,26 +426,33 @@ async def _send_dbus_message(
     signature: str,
     body: Any,
     negotiate_unix_fd: bool = False,
+    bus: MessageBus | None = None,
 ) -> tuple[MessageBus | None, Message | None]:
     """
     Private method to send messages to dbus via dbus_next.
 
+    An existing bus connection can be passed, if left empty, a new
+    bus connection will be created.
+
     Returns a tuple of the bus object and message response.
     """
-    if session_bus:
-        bus_type = BusType.SESSION
-    else:
-        bus_type = BusType.SYSTEM
+    if bus is None:
+        if session_bus:
+            bus_type = BusType.SESSION
+        else:
+            bus_type = BusType.SYSTEM
+
+        try:
+            bus = await MessageBus(
+                bus_type=bus_type, negotiate_unix_fd=negotiate_unix_fd
+            ).connect()
+        except (AuthError, Exception):
+            logger.warning("Unable to connect to dbus.")
+            return None, None
 
     if isinstance(body, str):
         body = [body]
 
-    try:
-        bus = await MessageBus(bus_type=bus_type, negotiate_unix_fd=negotiate_unix_fd).connect()
-    except (AuthError, Exception):
-        logger.warning("Unable to connect to dbus.")
-        return None, None
-
     # Ignore types here: dbus-next has default values of `None` for certain
     # parameters but the signature is `str` so passing `None` results in an
     # error in mypy.
@@ -459,6 +468,11 @@ async def _send_dbus_message(
         )
     )
 
+    # Keep details of bus connections so we can close them on exit
+    # dbus_bus_connetions is a set so we don't need to worry about
+    # duplicates
+    dbus_bus_connections.add(bus)
+
     return bus, msg
 
 
@@ -494,14 +508,16 @@ async def add_signal_receiver(
             return False
 
     match_args = {
-        "type": "signal",
         "sender": bus_name,
         "member": signal_name,
         "path": path,
         "interface": dbus_interface,
     }
 
-    rule = ",".join("{}='{}'".format(k, v) for k, v in match_args.items() if v)
+    rule = "type='signal',"
+    rule += ",".join("{}='{}'".format(k, v) for k, v in match_args.items() if v)
+
+    logger.debug("Adding dbus match rule: %s", rule)
 
     bus, msg = await _send_dbus_message(
         session_bus,
@@ -511,12 +527,54 @@ async def add_signal_receiver(
         "/org/freedesktop/DBus",
         "AddMatch",
         "s",
-        rule,
+        [rule],
     )
 
     # Check if message sent successfully
     if bus and msg and msg.message_type == MessageType.METHOD_RETURN:
-        bus.add_message_handler(callback)
+
+        def match_message(msg: Message, match_args: dict[str, str | None]) -> bool:
+            return msg._matches(**{k: v for k, v in match_args.items() if v})
+
+        async def resolve_sender(signal_msg: Message) -> tuple[str, Message]:
+            """Looks up a pretty bus name to retrieve the unique name."""
+            _, sender_msg = await _send_dbus_message(
+                session_bus,
+                MessageType.METHOD_CALL,
+                "org.freedesktop.DBus",
+                "org.freedesktop.DBus",
+                "/org/freedesktop/DBus",
+                "GetNameOwner",
+                "s",
+                [match_args["sender"]],
+                bus=bus,
+            )
+
+            if sender_msg and sender_msg.message_type == MessageType.METHOD_RETURN:
+                return sender_msg.body[0], signal_msg
+
+            return "", signal_msg
+
+        def check_message(task: asyncio.Task) -> None:
+            new_match_args = match_args.copy()
+            new_sender, signal_message = task.result()
+            new_match_args["sender"] = new_sender
+            if match_message(signal_message, new_match_args):
+                callback(signal_message)
+
+        def signal_callback_wrapper(msg: Message) -> None:
+            """Custom wrapper to only run callback if message matches our rule."""
+            if msg.message_type == MessageType.SIGNAL:
+                if match_message(msg, match_args):
+                    callback(msg)
+                elif "sender" in match_args:
+                    # If the message didn't match and we're trying to match the sender
+                    # We may need to convert the pretty name to the bus's unique name first
+                    task = create_task(resolve_sender(msg))
+                    if task:
+                        task.add_done_callback(check_message)
+
+        bus.add_message_handler(signal_callback_wrapper)
         return True
 
     else:
@@ -548,3 +606,18 @@ async def find_dbus_service(service: str, session_bus: bool) -> bool:
     names = msg.body[0]
 
     return service in names
+
+
+def remove_dbus_rules() -> None:
+    # Disconnecting the bus connections is enough to remove the match rules.
+    while dbus_bus_connections:
+        bus = dbus_bus_connections.pop()
+        try:
+            bus.disconnect()
+        except OSError:
+            # Socket has already shut down
+            pass
+
+        # We need to manually close the socket until https://github.com/altdesktop/python-dbus-next/pull/148
+        # gets merged. There's no error on multiple calls to 'close()'.
+        bus._sock.close()
